@@ -366,8 +366,8 @@ class VehicleDynamics(ABC):
         :param orientation: orientation
         :return: velocity_x, velocity_y
         """
-        velocity_x = np.cos(orientation) * velocity
-        velocity_y = np.sin(orientation) * velocity
+        velocity_x = math.cos(orientation) * velocity
+        velocity_y = math.sin(orientation) * velocity
         return velocity_x, velocity_y
 
 
@@ -485,8 +485,8 @@ class KinematicSingleTrackDynamics(VehicleDynamics):
     def _state_to_array(self, state: State, steering_angle_default=0.0) -> Tuple[np.array, int]:
         """ Implementation of the VehicleDynamics abstract method. """
         values = [
-            state.position[0],
-            state.position[1],
+            state.position[0] - self.parameters.b * math.cos(state.orientation),
+            state.position[1] - self.parameters.b * math.sin(state.orientation),
             getattr(state, 'steering_angle', steering_angle_default),  # not defined in initial state
             state.velocity,
             state.orientation
@@ -497,7 +497,8 @@ class KinematicSingleTrackDynamics(VehicleDynamics):
     def _array_to_state(self, x: np.array, time_step: int) -> State:
         """ Implementation of the VehicleDynamics abstract method. """
         values = {
-            'position': np.array([x[0], x[1]]),
+            'position': np.array([x[0] + self.parameters.b * math.cos(x[4]),
+                                  x[1] + self.parameters.b * math.sin(x[4])]),
             'steering_angle': x[2],
             'velocity': x[3],
             'orientation': x[4],
