@@ -11,7 +11,10 @@
 
 #ifndef ENABLE_TRIANGULATION
 #define ENABLE_TRIANGULATION 1
-#define ENABLE_TRIANGLE 1
+#endif
+
+#ifndef ENABLE_GPC
+#define ENABLE_GPC 1
 #endif
 
 #ifndef ENABLE_POLYGON_VALIDITY_CHECKS
