@@ -32,6 +32,8 @@
 #include <pybind11/eigen.h>
 #include <pybind11/pybind11.h>
 #include <pybind11/stl.h>
+//#include <pybind11/embed.h>
+
 
 #include <Eigen/Dense>
 
@@ -1161,16 +1163,82 @@ void init_module_collision(py::module &m) {
            py::arg("outer_boundary"), py::arg("holes"),
            py::arg("triangle mesh"))
 #if ENABLE_TRIANGULATION
-      .def(py::init([](std::vector<std::array<double, 2>> outer_boundary,
-                       double mesh_quality_triangle, double mesh_quality_cgal) {
+
+	  .def(py::init([](std::vector<std::array<double, 2>> outer_boundary, std::vector<std::vector<std::array<double, 2>>> holes) {
+		std::vector<Eigen::Vector2d> vertices;
+		for (const auto &vertex : outer_boundary) {
+		  vertices.push_back(Eigen::Vector2d(vertex[0], vertex[1]));
+		}
+
+		std::vector<std::vector<Eigen::Vector2d>> hole_vertices;
+
+
+		 for (const auto &hole : holes) {
+		   std::vector<Eigen::Vector2d> hole_vert;
+
+		   for (const auto &hole_vt : hole) {
+			 hole_vert.push_back(Eigen::Vector2d(hole_vt[0], hole_vt[1]));
+		   }
+		   hole_vertices.push_back(hole_vert);
+		 }
+
+		return new collision::Polygon(
+			vertices, hole_vertices, 0, collision::triangulation::TriangulationQuality());
+	  }))
+
+      .def(py::init([](std::vector<std::array<double, 2>> outer_boundary, std::vector<std::vector<std::array<double, 2>>> holes, int triangulation_method,
+                       double mesh_quality) {
         std::vector<Eigen::Vector2d> vertices;
+
+        /*
+        if(triangulation_method==1)
+        {
+        	using namespace pybind11::literals;
+
+
+            py::module_ np = py::module_::import("numpy");
+
+    		py::object np_equal = np.attr("equal");
+    		auto test0=np_equal(outer_boundary[0],outer_boundary[-1]);
+    		auto locals = py::dict("test0"_a=test0,"outer_boundary"_a=outer_boundary);
+    		py::exec(R"(
+    			test1 = all(test0)
+    		)", py::globals(), locals);
+    		bool test1=locals["test1"].cast<bool>();
+    		if(test1)
+    		{
+    			py::exec(R"(
+    			    			verts = outer_boundary[:-1]
+    			    		)", py::globals(), locals);
+    		}
+    		else
+    		{
+    			py::exec(R"(
+    			                verts = outer_boundary
+    			    	 )", py::globals(), locals);
+    		}
+
+        }
+        */
+
         for (const auto &vertex : outer_boundary) {
           vertices.push_back(Eigen::Vector2d(vertex[0], vertex[1]));
         }
+        std::vector<std::vector<Eigen::Vector2d>> hole_vertices;
+
+
+		 for (const auto &hole : holes) {
+		   std::vector<Eigen::Vector2d> hole_vert;
+
+		   for (const auto &hole_vt : hole) {
+			 hole_vert.push_back(Eigen::Vector2d(hole_vt[0], hole_vt[1]));
+		   }
+		   hole_vertices.push_back(hole_vert);
+		 }
 
         return new collision::Polygon(
-            vertices, collision::triangulation::TriangulationQuality(
-                          mesh_quality_cgal, mesh_quality_triangle));
+            vertices, hole_vertices, triangulation_method, collision::triangulation::TriangulationQuality(
+                          mesh_quality));
       }))
 #endif
       .def("collide",
